import pandas as pd

import plotly.graph_objects as go

from sklearn.feature_selection import SelectKBest, f_regression, VarianceThreshold

def select_k_best(X, y, k):
    variances = pd.DataFrame({'Features': X.columns, 'Variances': X.var()})
    feats = variances[variances['Variances'] > 0].Features.values
    X = X[feats]

    skb = SelectKBest(f_regression, k=k)
    df_skb = skb.fit_transform(X, y)

    feat_importances = pd.Series(skb.scores_, index=X.columns)

    importances = feat_importances.nlargest(k).sort_values().reset_index()
    importances.rename(columns={'index':'features', 0:'scores'}, inplace=True)

    return importances

def plot_best_features(feature_scores_df):
    fig = go.Figure(data = go.Bar(x=feature_scores_df['scores'],
                              y=feature_scores_df['features'],
                              orientation='h'))
    fig.update_layout(yaxis={ 'tickmode':'linear'})
    fig.show()