import numpy as np
import pandas as pd

import joblib
import itertools

def preprocess_data(df):
    df =  df.loc[:,('brand', 'model', 'year_prod', 'hp', 'mileage',
                    'views', 'color', 'engine_type', 'gearbox', 'category', 'dealer',
                    'city', 'region', 'gps', 'auto_start_stop', 'xenon_lights', 'alloy_wheels',
                    'metalic')]

    df['views'] = np.log(df['views'])

    df.replace({'Да':1, 'Не':0}, inplace=True)

    preproc = joblib.load('./model/preprocessor.pickle')

    df_transformed = preproc.fit_transform(df).toarray()

    categorical_features = ['brand', 'model', 'color', 'engine_type',
                            'gearbox', 'category','dealer', 'city', 'region']

    cols_list_new = [preproc.transformers_[0][2],
                     preproc.transformers_[1][1].get_feature_names(categorical_features),
                     preproc.transformers_[2][2]]
    
    cols_new = list(itertools.chain.from_iterable(cols_list_new))

    df_coded = pd.DataFrame(df_transformed, columns=cols_new)

    final_features = ['xenon_lights', 'hp', 'year_prod']

    missing_cols = set(final_features) - set(df_coded.columns)
    mutual_cols = set(final_features) & set(df_coded.columns)

    df_coded = df_coded[mutual_cols]

    for feature in missing_cols:
        df_coded[feature] = [0] * len(df_coded)
    
    return df_coded


